"""
VendAsta Package Manager
"""
import os
import sys
import base64
import json
import logging
import getpass

from github import Github, GithubException

CONFIG_FILE = 'vpmrc.json'
CONFIG_USERNAME = 'githubUsername'
CONFIG_PASSWORD = 'githubPassword'
PACKAGE_FILE = 'packages.json'
CACHE_ROOT = '.vpmcache'
INSTALL_PATH = 'installPath'
PACKAGES = 'packages'


def load_configuration(vpmrc_file, file_open=open):
    """
    Loads the VPM configuration.
    """
    try:
        config_file = file_open(vpmrc_file, "r")
        json_data = config_file.read()
        data = json.loads(json_data)
        if not CONFIG_USERNAME in data:
            logging.critical("githubUsername is a required \
                    field in vpmrc.json.")
            raise ValueError('Username missing in vpmrc.json')

        if not CONFIG_PASSWORD in data:
            logging.critical("githubPassword is a required \
                    field in vpmrc.json.")
            raise ValueError('Username missing in vpmrc.json')

        username = data[CONFIG_USERNAME]
        password = data[CONFIG_PASSWORD]

        config_file.close()
        return username, password
    except (IOError, ValueError):
        username = raw_input("Github username: ")
        password = getpass.getpass("Github password: ")
    finally:
        return username, password

def load_package_configuration(package_file, file_open=open):
    """
    Loads the package configuration.
    """
    try:
        config_file = file_open(package_file, "r")
        json_data = config_file.read()
        data = json.loads(json_data)
        config_file.close()
        return data
    except (IOError, ValueError) as exception:
        logging.critical("A package.json file was not found \
                or is ill formatted. %s", exception)
        sys.exit(1)


def find_package_in_configuration(package, package_config):
    """
    Returns the package entry in the package_config JSON data
    """
    for package_list in package_config:
        install_path, package_dict = load_package_repository(package_list)
        package_path = package_dict.get(package, None)
        if package_path:
            break

    if not package_path:
        logging.critical('The requested package is not in packages.json')
        sys.exit(1)

    return package_path, install_path


def load_package_repository(package_dict):
    """
    Returns the packages and install path in the dictionary.
    """
    if not INSTALL_PATH in package_dict:
        logging.critical('installPath variable must be set for each \
                package in package.json.')
        sys.exit(1)

    install_path = package_dict[INSTALL_PATH]
    packages = package_dict[PACKAGES]
    return install_path, packages


def get_repository_for_package(package_repository, username, password):
    """
    Returns a PyGithub repository object for the specified repository.
    """
    github = Github(username, password)

    organization = github.get_user().get_orgs()[0]
    repository = organization.get_repo(package_repository)

    return repository


def get_sha_for_tag(repository, tag):
    """
    Returns a commit PyGithub object for the specified repository and tag.
    """
    branches = repository.get_branches()
    matched_branches = [match for match in branches if match.name == tag]
    if matched_branches:
        return matched_branches[0].commit.sha

    tags = repository.get_tags()
    matched_tags = [match for match in tags if match.name == tag]
    if not matched_tags:
        raise ValueError('No Tag or Branch exists with that name')
    return matched_tags[0].commit.sha


def clear_cache():
    """
    Delete the contents of CACHE_ROOT.
    """
    if not os.path.exists(CACHE_ROOT):
        os.makedirs(CACHE_ROOT)

    os.popen('rm -rf ' + CACHE_ROOT + '/*')


def download_package(repository, sha, server_path, download_path):
    """
    Download all contents at server_path with commit tag sha in 
    the repository.
    """
    contents = repository.get_dir_contents(server_path, ref=sha)

    for content in contents:
        print "Processing %s" % content.path
        if content.type == 'dir':
            import pdb; pdb.set_trace()
            dir_download_path = '%s/%s' % (download_path, content.name)
            os.makedirs(dir_download_path)
            download_package(repository, sha, content.path, dir_download_path)
        else:
            try:
                path = content.path
                file_content = repository.get_contents(path, ref=sha)
                file_data = base64.b64decode(file_content.content)
                file_out = open(download_path + '/' + content.name, "w")
                file_out.write(file_data)
                file_out.close()
            except (GithubException, IOError) as exc:
                logging.error('Error processing %s: %s', content.path, exc)


def vpm(package, tag=None):
    """
    VendAsta Package Manager.
    """
    # Load configuration
    print "Loading configuration"
    username, password = load_configuration(CONFIG_FILE)
    package_config = load_package_configuration(PACKAGE_FILE)

    full_package_path, install_path = \
            find_package_in_configuration(package, package_config)

    [package_repository, package_server_path] = full_package_path.split('/', 1)

    # GitHub connection
    print "Connecting to GitHub"
    repository = get_repository_for_package(package_repository, 
            username, password)
    sha = get_sha_for_tag(repository, tag)

    clear_cache()

    # Download package
    print "Downloading package contents"
    package_download_path = CACHE_ROOT + '/' + package

    if not os.path.exists(package_download_path):
        os.makedirs(package_download_path)

    download_package(repository, sha, package_server_path, 
            package_download_path)

    # Install package
    print "Removing existing package"
    package_install_path = install_path + '/' + package

    os.popen('rm -rf ' + package_install_path)

    if not os.path.exists(package_install_path):
        os.makedirs(package_install_path)

    print "Installing %s version %s" % (package, tag)
    os.popen('mv ' + package_download_path + '/*' + ' ' + package_install_path)


def run():
    """
    Main entry point.
    """
    args = sys.argv[1:]
    if not len(args) == 2:
        logging.error("Usage: vpm package version")
        return
    vpm(args[0], args[1])


if __name__ == '__main__':
    run()
